/* PrettyOS is the result of a stepwise OS development
 * shown at www.henkessoft.de */

#ifndef OS_H
#define OS_H

#define TRUE		1
#define FALSE		0

typedef unsigned int   size_t;

typedef unsigned int   UINT;
typedef unsigned char  UCHAR;
typedef unsigned short USHORT;
typedef unsigned long  ULONG;
typedef signed char    CHAR;

extern ULONG placement_address;

#define ASSERT(b) ((b) ? (void)0 : panic_assert(__FILE__, __LINE__, #b))
extern void panic_assert(const char* file, ULONG line, const char* desc);

/* This defines the operatings system common data area */

#define KQSIZE    20 // size of key queue

typedef struct oda
{
    // Hardware Data
    ULONG COM1, COM2, COM3, COM4; // address
    ULONG LPT1, LPT2, LPT3, LPT4; // address
    ULONG Memory_Size;            // Memory size in Byte

    // Key Queue
    UCHAR  KEYQUEUE[KQSIZE];   // circular queue buffer
    UCHAR* pHeadKQ;            // pointer to the head of valid data
    UCHAR* pTailKQ;            // pointer to the tail of valid data
    ULONG  KQ_count_read;      // number of data read from queue buffer
    ULONG  KQ_count_write;     // number of data put into queue buffer

    //tasking
    UCHAR  ts_flag;            // 0: taskswitch off  1: taskswitch on
}oda_t;

// operatings system common data area
oda_t ODA;
extern oda_t* pODA;

/* This defines what the stack looks like after an ISR was running */
typedef struct regs
{
    ULONG gs, fs, es, ds;
    ULONG edi, esi, ebp, /*esp,*/ ebx, edx, ecx, eax;
    ULONG int_no, err_code;
    ULONG eip, cs, eflags, useresp, ss;
}registers_t;

// video.c
extern void k_clear_screen();
extern void settextcolor(UCHAR forecolor, UCHAR backcolor);
extern void putch(UCHAR c);
extern void puts(UCHAR* text);
extern void scroll();
extern void k_printf(UCHAR* message, UINT line, UCHAR attribute);
extern void set_cursor(UCHAR x, UCHAR y);
extern void update_cursor();
extern void move_cursor_right();
extern void move_cursor_left();
extern void move_cursor_home();
extern void move_cursor_end();
extern void save_cursor();
extern void restore_cursor();
extern void printformat (char *args, ...);

// timer.c
extern void timer_handler(struct regs* r);
extern void timer_wait (ULONG ticks);
extern void sleepSeconds (ULONG seconds);
extern void sleepMilliSeconds (ULONG ms);
extern void systemTimer_setFrequency( ULONG freq );
extern void timer_install();
extern void timer_uninstall();

// keyboard.c
extern void keyboard_init();
extern UCHAR FetchAndAnalyzeScancode();
extern UCHAR ScanToASCII();
extern void keyboard_handler(struct regs* r);
extern int k_checkKQ_and_print_char();

// util.c
extern void initODA();
extern void outportb(UINT port, UINT val);
extern UINT inportb(UINT port);
extern ULONG fetchESP();
extern ULONG fetchEBP();
extern ULONG fetchSS();
extern ULONG fetchCS();
extern ULONG fetchDS();
extern void k_memshow(void* start, size_t count);
extern void* k_memset(void* dest, char val, size_t count);
extern USHORT* k_memsetw(USHORT* dest, USHORT val, size_t count);
extern void* k_memcpy(void* dest, const void* src, size_t count);
extern size_t k_strlen(const char* str);
extern int k_strcmp(char* str1, char* str2);
extern char* k_strcpy(char* dest, const char* src);
extern char* k_strcat(char* dest, const char* src);
extern void reboot();
extern void cli();
extern void sti();
extern void k_itoa(int value, char* valuestring);
extern void k_i2hex(UINT val, UCHAR* dest, int len);
extern void float2string(float value, int decimal, char* valuestring);

// gtd.c itd.c irq.c
extern void gdt_set_gate(int num, ULONG base, ULONG limit, UCHAR access, UCHAR gran);
extern void gdt_install();
extern void idt_set_gate(UCHAR num, ULONG base, USHORT sel, UCHAR flags);
extern void idt_install();
extern void isrs_install();
extern void irq_install();

extern void timer_install();
extern void keyboard_install();

extern void fault_handler(struct regs* r);
extern void irq_install_handler(int irq, void (*handler)(struct regs* r));
extern void irq_uninstall_handler(int irq);
extern void irq_remap(void);
extern void irq_handler(struct regs* r);
extern ULONG irq_handler1(ULONG esp);

// math.c
extern int k_abs(int i);
extern int k_power(int base,int n);

/*
extern void beep();
extern void sound(UINT frequency);
extern void noSound();
*/

// paging.c
extern void paging_install();
extern ULONG k_malloc(ULONG size, UCHAR align, ULONG* phys);
extern void analyze_frames_bitset(ULONG sec);
extern ULONG show_physical_address(ULONG virtual_address);
extern void analyze_physical_addresses();

// initrd.c

// task.c
//extern void tasking_install();
//extern void switch_to_user_mode();

// syscall.c
extern void syscall_handler(struct regs* regs);

//process.asm
extern ULONG file_data_start;
extern ULONG file_data_end;

#endif
